@@ -19,6 +19,10 @@ generateRecombinationDistances_noInterference = function ( n ) {
 
 
 
+
+
+
+
 #' Genetates a recombination vector arising from one meiotic event.
 #' The origin of segments is coded as (0 - haplotype1 ,  1 - haplotype2 )
 #'
@@ -29,7 +33,7 @@ generateRecombinationDistances_noInterference = function ( n ) {
 #' library("sim1000G")
 #'
 #' examples_dir = system.file("examples", package = "sim1000G")
-#' vcf_file = sprintf("%s/region.vcf.gz", examples_dir)
+#' vcf_file = file.path(examples_dir, "region.vcf.gz")
 #' vcf = readVCF( vcf_file, maxNumberOfVariants = 100 , min_maf = 0.12 ,max_maf = NA)
 #'
 #' # For realistic data use the functions downloadGeneticMap / readGeneticMap
@@ -77,6 +81,35 @@ generateSingleRecombinationVector = function(cm) {
 
 
 
+#' Genetates a recombination vector arising from one meiotic event.
+#' The origin of segments is coded as (0 - haplotype1 ,  1 - haplotype2 )
+#'
+#' @param chromosomeLength The length of the region in cm.
+#'
+#' @examples
+#'
+#' library("sim1000G")
+#'
+#' readGeneticMap(4)
+#' generateChromosomeRecombinationPositions(500)
+#'
+#' @export
+generateChromosomeRecombinationPositions = function(chromosomeLength = 500) {
+
+    pos = generateRecombinationDistances(10)
+
+    while(sum(pos) < chromosomeLength) pos = c(pos, generateRecombinationDistances(10) )
+
+    pos = cumsum(pos)
+
+    pos = pos[pos <= chromosomeLength]
+    pos
+}
+
+
+
+
+
 
 
 
@@ -93,17 +126,18 @@ geneticMap <- new.env()
 #' Downloads a genetic map for a particular chromosome under GRCh37 coordinates for use with sim1000G.
 #'
 #' @param chromosome Chromosome number to download recombination distances from.
+#' @param dir Directory to save the genetic map to (default: extdata)
 #'
 #'
 #' @examples
 #'
-#' \dontrun{
 #'
-#' downloadGeneticMap(22)
 #'
-#' }
+#' downloadGeneticMap(22, dir=tempdir() )
+#'
+#'
 #' @export
-downloadGeneticMap = function(chromosome) {
+downloadGeneticMap = function(chromosome, dir = NA) {
 
 
         fname = sprintf("genetic_map_GRCh37_chr%s.txt.gz" , chromosome )
@@ -113,52 +147,78 @@ downloadGeneticMap = function(chromosome) {
             chromosome
         )
 
-        dest_dir = system.file("data", fname, package = "sim1000G")
-        dest_dir = "."
+        # dest_dir = system.file("extdata", fname, package = "sim1000G")
 
-        cat("Downloading genetic map\n")
-        dest_dir = system.file("data", package = "sim1000G")
 
-        dest_dir = "./"
-        dest_path = sprintf("%s/genetic_map_GRCh37_chr%s.txt.gz", dest_dir, chromosome)
+
+        if(is.na(dir)) {
+            dest_dir = system.file("datasets", package = "sim1000G")
+
+            if(dest_dir == "") dest_dir = tempdir()
+            if(!dir.exists(dest_dir))  dest_dir = tempdir()
+        } else {
+            dest_dir = dir
+        }
+
+
+        cat("yes..\n", dest_dir, file="/tmp/xxx")
+
+        #dest_dir = "./"
+        dest_path = file.path(dest_dir, fname)
+
+
+        if(file.exists(dest_path)) {
+            cat(" -> Using genetic map found on ", dest_path, "\n");
+            return(dest_path)
+
+        }
+
+        cat(" -> Downloading genetic map from:",url,"\n")
         cat(" -> Saving genetic map to: " , dest_path,"\n")
-        #file.exists(dest_path)
 
         download.file(url  ,  destfile = dest_path, quiet=TRUE)
-
+        return(dest_path)
 
 }
 
 
 
 
-#' Reads a genetic map downloaded from the function downloadGeneticMap.
+
+#' Reads a genetic map downloaded from the function downloadGeneticMap or reads a genetic map from a specified file. If the argument filename is used
+#' then the genetic map is read from the corresponding file. Otherwise, if a chromosome is specified, the genetic map is downloaded for human chromosome
+#' using grch37 coordinates.
 #'
 #' The map must contains a complete chromosome or enough markers to cover the area that
 #' will be simulated.
 #'
-#' The file can be downloaded from the provided genetic maps by using the function downloadGeneticMap.
 #'
-#'
-#' @param chromosome Chromosome number to download recombination distances from.
-#' @param dir Directory the map file is located.
+#' @param chromosome Chromosome number to download a genetic map for , or
+#' @param filename A filename of an existing genetic map to read from (default NA).
+#' @param dir Directory the map file will be saved (only if chromosome is specified).
 #'
 #'
 #'
 #' @examples
 #'
-#' \dontrun{
 #'
-#' downloadGeneticMap(22)
-#' readGeneticMap(22)
 #'
-#' }
+#'
+#' readGeneticMap(chromosome = 22)
+#'
+#'
 #'
 #' @export
-readGeneticMap = function(chromosome, dir=".") {
+readGeneticMap = function(chromosome, filename=NA, dir=NA) {
 
-    filelocation = sprintf( "%s/genetic_map_GRCh37_chr%s.txt.gz", dir, as.character( chromosome) )
-    readGeneticMapFromFile(filelocation)
+
+    if( ! is.na(filename) ) { return ( readGeneticMapFromFile(filename) ) }
+    #if(is.na(dir)) dir = system.file("extdata", package = "sim1000G")
+
+    fname =   downloadGeneticMap( chromosome ,dir=dir)
+
+
+    readGeneticMapFromFile(fname)
 
 }
 
@@ -200,15 +260,14 @@ readGeneticMapFromFile = function(filelocation) {
     cat("      -> Genetic map has" , length(geneticMap$bp), "entries\n");
 
     makeCDF()
-    0
 }
 
 
 
-#' Generates a fake genetic map.
+#' Generates a uniform genetic map.
 #'
 #'
-#' Generates a fake genetic map by approximating 1 cm / Mbp. Only used for examples.
+#' Generates a uniform genetic map by approximating 1 cm / Mbp. Only used for examples.
 #'
 #'
 #' @examples
@@ -241,6 +300,42 @@ generateUniformGeneticMap = function() {
 
 
 
+#' Generates a fake genetic map that spans the whole genome.
+#'
+#' @examples
+#'
+#' library("sim1000G")
+#'
+#' examples_dir = system.file("examples", package = "sim1000G")
+#' vcf_file = sprintf("%s/region.vcf.gz", examples_dir)
+#' vcf = readVCF( vcf_file, maxNumberOfVariants = 100 , min_maf = 0.12 ,max_maf = NA)
+#'
+#' # For realistic data use the functions downloadGeneticMap / readGeneticMap
+#' generateFakeWholeGenomeGeneticMap(vcf)
+#' plotRegionalGeneticMap(seq(1e6,100e6,by=1e6/2))
+#'
+#' @param vcf A vcf file read by function readVCF.
+#' @export
+generateFakeWholeGenomeGeneticMap = function(vcf) {
+
+    n = 10000
+    bp = vcf$vcf[,2]
+    cm = seq(0,4000, l=length(bp))
+
+    geneticMap$chr = rep(4, length(bp))
+    geneticMap$bp = bp
+    geneticMap$cm = cm
+
+    makeCDF();
+    0;
+
+}
+
+
+
+
+
+
 
 #' Converts centimorgan position to base-pair. Return a list of centimorgan positions that correspond
 #' to the bp vector (in basepairs).
@@ -291,8 +386,12 @@ plotRegionalGeneticMap = function(bp) {
     #bp = vcf[,2]
     cm = approx( geneticMap$bp, geneticMap$cm, bp )$y
 
+
+    len = diff(range(cm))
+    print(len)
+
     plot(bp/1e6,cm, t="l",cex=0.2, xlab="MBp", ylab = "Centimorgan")
-    segments( bp/1e6,cm,  bp/1e6,cm -0.2, lwd=0.3,col="blue")
+    segments( bp/1e6,cm,  bp/1e6,cm - len/10, lwd=0.3,col="blue")
 
 
 }
@@ -301,7 +400,6 @@ plotRegionalGeneticMap = function(bp) {
 
 
 
-
 #' Contains recombination model information.
 #'
 #' This vector contains the density between two recombination events, as a cumulative density function.
@@ -356,17 +454,32 @@ makeCDF = function() {
 
 
 
-#' Generate recombination distances using a chi-square model.
+#' Generate inter-recombination distances using a chi-square model. Note this are the distances between two succesive recombination events and not
+#' the absolute positions of the events. To generate the locations of the recombination events see the example below.
 #'
 #'
 #' @param n Number of distances to generate
 #'
-#' @return vector of recombination distances in centimorgan
+#' @return vector of distances between two recombination events.
+#'
+#' @examples
+#'
+#' library("sim1000G")
+#'
+#' distances = generateRecombinationDistances(20)
+#'
+#'
+#' positions_of_recombination = cumsum(distances)
+#'
+#'
 #'
 #' @export
 generateRecombinationDistances = function ( n ) {
 
 
+    if(pkg.opts$recombination == 0) { return(generateRecombinationDistances_noInterference(n))}
+
+
     t = findInterval( runif(n), crossoverCDF$vector )
     t = t * 4 / length(crossoverCDF$vector)
     100 * t
